@@ -1,7 +1,7 @@
 __doc__ = """
 This program requires python 3.6 or higher.
 
-Print(functionName.__doc__) to see the 
+Print(functionName.__doc__) to see the
 
 documentation for the function or usage.
 
@@ -14,33 +14,33 @@ from math import sqrt, acos,degrees, pi
 def calcDistance(m, n):
     """
     This function calculates the distance between two points
-    
+
     Arguments
     ----------
-    m:   the first 3D point 
+    m:   the first 3D point
     n:   the second 3D point
-    
+
     Returns
     -------
     Float: the calculated distance
     """
-    return (sqrt(sum([(m[z] - n[z])**2 for z in range(3)]))) 
+    return (sqrt(sum([(m[z] - n[z])**2 for z in range(3)])))
 
 def calcCoordinate(a, b, c, bond_len, theta, di_angle):
     """
     Given 3 known atom coordinates this function
     obtains the 4th atom coordinate that satisfies
     the input geometry constraints.
-    
+
     Arguments
     ----------
-    a:        the first 3D point 
+    a:        the first 3D point
     b:        the second 3D point
     c:        the third 3D point
     bond_len: the bond length of c and the 4th atom
     theta:    the angle formed by b, c & the 4th atom
     di_angle: the dihedral angle defining the 4 atoms
-    
+
     Returns
     -------
     A list: the calculated 4th atom coordinate
@@ -79,14 +79,14 @@ def calcTorsionAngle(coord1, coord2, coord3, coord4):
     """
     Given 4 known atom coordinates this function
     calculates the dihedral angle defining the atoms
-    
+
     Arguments
     ----------
-    coord1: the first 3D point 
+    coord1: the first 3D point
     coord2: the second 3D point
     coord3: the third 3D point
     coord4: the fourth 3D point
-    
+
     Returns
     -------
     Float: the calculated angle in degrees
@@ -110,20 +110,20 @@ def calcTorsionAngle(coord1, coord2, coord3, coord4):
 
 def get_chi(atom_1, atom_2, atom_3, atom_4, resNum, chainNum,faspr):
     """
-    This function calculates flexible dihedral angle (chi) 
+    This function calculates flexible dihedral angle (chi)
     if a PDB output obtained from FASPR is supplied, if not
     this function is never used!
-    
+
     Arguments
     ----------
-    atom_1:   the first atom name 
-    atom_2:   the second atom name 
-    atom_3:   the third atom name 
+    atom_1:   the first atom name
+    atom_2:   the second atom name
+    atom_3:   the third atom name
     atom_4:   the fourth atom name
     resNum:   the residue number
     chainNum: the chain number
     faspr   : the chain as obtained from ReadMaster.py
-    
+
     Returns
     -------
     Float: the calculated angle in degrees
@@ -145,7 +145,7 @@ def addBackbone(N, CA, C, atmpos, psi):
     This function calculates the backbone O coordinate
     and then checks for van der Waals conflict. If there is conflict,
     it uses next favourable di angle until VDW conflict is resolved
-    
+
     Arguments
     ----------
     N:      the N atom coordinate
@@ -153,7 +153,7 @@ def addBackbone(N, CA, C, atmpos, psi):
     C:      the C atom coordinate
     atmpos: the amino acid that may clash with O
     psi   : the coordinates of atoms required to calculate psi
-    
+
     Returns
     -------
     A list: the calculated 3D coordinate of atom O
@@ -165,14 +165,14 @@ def addBackbone(N, CA, C, atmpos, psi):
         for i in di:
             O = calcCoordinate(N,CA,C,1.23,120.5,i)
             if calcDistance(O,atmpos[0]) > 2.0:
-                break   
-                                     
+                break
+
     elif psi > 0 and psi < 100:
         di = [-140,140,-30,40]
         for i in di:
             O = calcCoordinate(N,CA,C,1.23,120.5,i)
             if calcDistance(O,atmpos[0]) > 2.0:
-                break 
+                break
     else:
         di = [140,-140,-30,40]
         for i in di:
@@ -214,17 +214,17 @@ def ctermini(atom,pos):
 def repairSer(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
     This function plus the rest represent 20 amino acid residues.
-    Annotation is provided in detail here only  b/c all the 
-    functions are essentially the same, that is, they take 
+    Annotation is provided in detail here only  b/c all the
+    functions are essentially the same, that is, they take
     similar arguments except Ala and Gly that have no side-chain.
 
-    The first priority of PRAS is to keep resolved atoms and replace 
-    only the missing ones. Each atom that requires flexible chi to fix 
-    is indicated. If such atom is missing, all other atoms are fixed 
-    using chi supplied by FASPR or default PRAS chi taken from literature 
-    if FASPR PDB is not provided. If atoms that are missing do not require 
+    The first priority of PRAS is to keep resolved atoms and replace
+    only the missing ones. Each atom that requires flexible chi to fix
+    is indicated. If such atom is missing, all other atoms are fixed
+    using chi supplied by FASPR or default PRAS chi taken from literature
+    if FASPR PDB is not provided. If atoms that are missing do not require
     flexible chi, FASPR is not needed. All constraints (bond/di angle, bond
-    length are taken from Dunbrack 2011 rotamer library. 
+    length are taken from Dunbrack 2011 rotamer library.
 
     Arguments
     ----------
@@ -280,7 +280,7 @@ def repairSer(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairAla(atoms,coord,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the ALA residue.
     See repairSer() for full annotation
     """
@@ -303,7 +303,11 @@ def repairAla(atoms,coord,nextres='',psi=''):
     return [atoms, atompos]
 
 def repairGly(atoms,coord,nextres='',psi=''):
-
+    """
+    This function adds missing [O]
+    atom of the GLY residue.
+    See repairSer() for full annotation
+    """
     N  = coord[atoms.index('N')]
     CA = coord[atoms.index('CA')]
     C  = coord[atoms.index('C')]
@@ -319,7 +323,7 @@ def repairGly(atoms,coord,nextres='',psi=''):
 
 def repairAsn(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the ASN residue.
     See repairSer() for full annotation
     """
@@ -367,7 +371,7 @@ def repairAsn(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     # so we just calculate it
     CG = coord[atoms.index('CG')]
     OD1 = coord[atoms.index('OD1')]
-    CA_CB_CG_OD1_diangle = calcTorsionAngle(CA,CB,CG,OD1) 
+    CA_CB_CG_OD1_diangle = calcTorsionAngle(CA,CB,CG,OD1)
     CA_CB_CG_ND2_diangle = 180.0 + CA_CB_CG_OD1_diangle
     ND2 = calcCoordinate(CA, CB, CG, 1.33,116.48, CA_CB_CG_ND2_diangle)
     atompos = [N, CA, C, O, CB, CG, OD1, ND2]
@@ -376,7 +380,7 @@ def repairAsn(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairGlu(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the GLU residue.
     See repairSer() for full annotation
     """
@@ -432,10 +436,10 @@ def repairGlu(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     atompos = [N, CA, C, O, CB, CG, CD, OE1, OE2]
     atoms   = ['N', 'CA', 'C', 'O', 'CB', 'CG', 'CD', 'OE1', 'OE2']
     return [atoms, atompos]
- 
+
 def repairAsp(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the ASP residue.
     See repairSer() for full annotation
     """
@@ -489,7 +493,7 @@ def repairAsp(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairGln(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the GLN residue.
     See repairSer() for full annotation
     """
@@ -545,7 +549,7 @@ def repairGln(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairLys(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the LYS residue.
     See repairSer() for full annotation
     """
@@ -578,7 +582,7 @@ def repairLys(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
         N_CA_CB_CG_diangle = -64.5
         CA_CB_CG_CD_diangle = -178.1
         CB_CG_CD_CE_diangle = -179.6
-        CG_CD_CE_NZ_diangle = 179.6 
+        CG_CD_CE_NZ_diangle = 179.6
 
     # for LYS, fixing CG,CD,CE,NZ require flexible chi
     require_chi = ['CG', 'CD', 'CE', 'NZ']
@@ -588,7 +592,7 @@ def repairLys(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
             CG = calcCoordinate(N, CA, CB, 1.52, 113.83, N_CA_CB_CG_diangle)
             CD = calcCoordinate(CA, CB, CG, 1.52, 111.79, CA_CB_CG_CD_diangle)
             CE = calcCoordinate(CB, CG, CD, 1.46, 111.68, CB_CG_CD_CE_diangle)
-            NZ = calcCoordinate(CG, CD, CE, 1.33, 124.79, CG_CD_CE_NZ_diangle) 
+            NZ = calcCoordinate(CG, CD, CE, 1.33, 124.79, CG_CD_CE_NZ_diangle)
             atompos = [N, CA, C, O, CB, CG, CD, CE, NZ]
             atoms   = ['N', 'CA', 'C', 'O', 'CB', 'CG', 'CD', 'CE', 'NZ']
             return [atoms, atompos]
@@ -603,7 +607,7 @@ def repairLys(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairArg(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the ARG residue.
     See repairSer() for full annotation
     """
@@ -667,7 +671,7 @@ def repairArg(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     try:
         NH2 = coord[atoms.index('NH2')]
     except:
-        NH2 = calcCoordinate(CD, NE, CZ, 1.33, 119.63, 180.0) 
+        NH2 = calcCoordinate(CD, NE, CZ, 1.33, 119.63, 180.0)
 
     atompos = [N, CA, C, O, CB, CG, CD, NE, CZ, NH1, NH2]
     atoms   = ['N','CA','C','O','CB','CG','CD','NE','CZ','NH1','NH2']
@@ -675,7 +679,7 @@ def repairArg(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairTyr(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the TYR residue.
     See repairSer() for full annotation
     """
@@ -715,7 +719,7 @@ def repairTyr(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
             CD1 = calcCoordinate(CA, CB, CG, 1.39, 120.98, CA_CB_CG_CD1_diangle)
             CD2 = calcCoordinate(CA, CB, CG, 1.39, 120.82,CA_CB_CG_CD2_diangle)
             CE1 = calcCoordinate(CB, CG, CD1, 1.39, 120.0,180.0)
-            CE2 = calcCoordinate(CB, CG, CD2, 1.39, 120.0, 180.0)     
+            CE2 = calcCoordinate(CB, CG, CD2, 1.39, 120.0, 180.0)
             CZ = calcCoordinate(CG, CD1, CE1, 1.39, 120.0, 0.0)
             OH = calcCoordinate(CD1, CE1, CZ, 1.39, 119.78, 180.0)
             atompos = [N, CA, C, O, CB, CG, CD1, CD2, CE1, CE2, CZ, OH]
@@ -731,12 +735,12 @@ def repairTyr(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
         CA_CB_CG_CD1_diangle = calcTorsionAngle(CA,CB,CG,CD1)
         CA_CB_CG_CD2_diangle = CA_CB_CG_CD1_diangle + 180.0
         CD2 = calcCoordinate(CA, CB, CG, 1.39, 120.82,CA_CB_CG_CD2_diangle)
-    try:   
+    try:
         CE1 = coord[atoms.index('CE1')]
     except:
         CE1 = calcCoordinate(CB, CG, CD1, 1.39, 120.0,180.0)
     try:
-        CE2 = coord[atoms.index('CE2')] 
+        CE2 = coord[atoms.index('CE2')]
     except:
         CE2 = calcCoordinate(CB, CG, CD2, 1.39, 120.0, 180.0)
     try:
@@ -754,7 +758,7 @@ def repairTyr(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairTrp(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the TRP residue.
     See repairSer() for full annotation
     """
@@ -797,10 +801,10 @@ def repairTrp(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
             CE2 = calcCoordinate(CB, CG, CD2, 1.40, 108.5, 180.0)
             CE3 = calcCoordinate(CB, CG, CD2, 1.40, 133.83, 0.0)
             CZ2 = calcCoordinate(CG, CD2, CE2, 1.40, 120.0, 180.0)
-            CZ3 = calcCoordinate(CG, CD2, CE3, 1.40, 120.0, 180.0)       
+            CZ3 = calcCoordinate(CG, CD2, CE3, 1.40, 120.0, 180.0)
             CH2 = calcCoordinate(CD2, CE2, CZ2, 1.40, 120.0, 0.0)
             atompos = [N, CA, C, O, CB, CG, CD1, CD2, NE1, CE2, CE3, CZ2, CZ3, CH2]
-            atoms   = ['N', 'CA', 'C', 'O', 'CB', 'CG', 'CD1', 'CD2', 'NE1', 'CE2', 
+            atoms   = ['N', 'CA', 'C', 'O', 'CB', 'CG', 'CD1', 'CD2', 'NE1', 'CE2',
                       'CE3', 'CZ2','CZ3', 'CH2']
             return [atoms, atompos]
 
@@ -836,7 +840,7 @@ def repairTrp(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     try:
         CH2 = coord[atoms.index('CH2')]
     except:
-        CH2 = calcCoordinate(CD2, CE2, CZ2, 1.40, 120.0, 0.0)   
+        CH2 = calcCoordinate(CD2, CE2, CZ2, 1.40, 120.0, 0.0)
 
     atompos = [N, CA, C, O, CB, CG, CD1, CD2, NE1, CE2, CE3, CZ2, CZ3, CH2]
     atoms   = ['N', 'CA', 'C', 'O', 'CB', 'CG', 'CD1', 'CD2', 'NE1', 'CE2', 'CE3', 'CZ2','CZ3', 'CH2']
@@ -844,7 +848,7 @@ def repairTrp(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairPhe(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the PHE residue.
     See repairSer() for full annotation
     """
@@ -910,7 +914,7 @@ def repairPhe(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     try:
         CZ = coord[atoms.index('CZ')]
     except:
-        CZ = calcCoordinate(CG, CD1, CE1, 1.39, 120.0, 0.0) 
+        CZ = calcCoordinate(CG, CD1, CE1, 1.39, 120.0, 0.0)
 
     atompos = [N, CA, C, O, CB, CG, CD1, CD2, CE1, CE2, CZ]
     atoms   = ['N', 'CA', 'C', 'O', 'CB', 'CG', 'CD1', 'CD2', 'CE1', 'CE2', 'CZ']
@@ -918,7 +922,7 @@ def repairPhe(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairHis(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the HIS residue.
     See repairSer() for full annotation
     """
@@ -954,10 +958,10 @@ def repairHis(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
     for i in missing:
         if i in require_chi:
-            CG = calcCoordinate(N, CA, CB, 1.49, 113.74, N_CA_CB_CG_diangle)            
+            CG = calcCoordinate(N, CA, CB, 1.49, 113.74, N_CA_CB_CG_diangle)
             ND1 = calcCoordinate(CA, CB, CG, 1.38, 122.85, CA_CB_CG_ND1_diangle)
             CD2 = calcCoordinate(CA, CB, CG, 1.35, 130.61, CA_CB_CG_CD2_diangle)
-            CE1 = calcCoordinate(CB, CG, ND1, 1.32, 108.5, 180.0)                                      
+            CE1 = calcCoordinate(CB, CG, ND1, 1.32, 108.5, 180.0)
             NE2 = calcCoordinate(CB, CG, CD2, 1.35, 108.5, 180.0)
             atompos = [N, CA, C, O, CB, CG, ND1, CD2, CE1, NE2]
             atoms   = ['N', 'CA', 'C', 'O', 'CB', 'CG', 'ND1', 'CD2', 'CE1', 'NE2']
@@ -977,7 +981,7 @@ def repairHis(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     try:
         NE2 = coord[atoms.index('NE2')]
     except:
-        NE2 = calcCoordinate(CB, CG, CD2, 1.35, 108.5, 180.0)  
+        NE2 = calcCoordinate(CB, CG, CD2, 1.35, 108.5, 180.0)
 
     atompos = [N, CA, C, O, CB, CG, ND1, CD2, CE1, NE2]
     atoms   = ['N', 'CA', 'C', 'O', 'CB', 'CG', 'ND1', 'CD2', 'CE1', 'NE2']
@@ -985,7 +989,7 @@ def repairHis(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairPro(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the PRO residue.
     See repairSer() for full annotation
     """
@@ -1033,7 +1037,7 @@ def repairPro(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairMet(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the MET residue.
     See repairSer() for full annotation
     """
@@ -1086,7 +1090,7 @@ def repairMet(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairIle(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the ILE residue.
     See repairSer() for full annotation
     """
@@ -1140,7 +1144,7 @@ def repairIle(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairLeu(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the LEU residue.
     See repairSer() for full annotation
     """
@@ -1163,12 +1167,12 @@ def repairLeu(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
             CA_CB_CG_CD1_diangle = get_chi( 'CA', 'CB', 'CG','CD1', resNo, chain,faspr)
         except:
             N_CA_CB_CG_diangle = -60.1
-            CA_CB_CG_CD1_diangle = 174.9 
+            CA_CB_CG_CD1_diangle = 174.9
 
     else:
         N_CA_CB_CG_diangle = -60.1
-        CA_CB_CG_CD1_diangle = 174.9 
-    
+        CA_CB_CG_CD1_diangle = 174.9
+
     CA_CB_CG_CD2_diangle = CA_CB_CG_CD1_diangle + 120
 
     # for LEU, fixing CG,CD1 require flexible chi
@@ -1194,7 +1198,7 @@ def repairLeu(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairVal(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the VAL residue.
     See repairSer() for full annotation
     """
@@ -1242,7 +1246,7 @@ def repairVal(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairThr(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the THR residue.
     See repairSer() for full annotation
     """
@@ -1291,7 +1295,7 @@ def repairThr(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
 
 def repairCys(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     """
-    This function adds missing heavy 
+    This function adds missing heavy
     atoms of the CYS residue.
     See repairSer() for full annotation
     """
@@ -1327,7 +1331,7 @@ def repairCys(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
             atoms   = ['N', 'CA', 'C', 'O', 'CB', 'SG']
             return [atoms, atompos]
 
-    SG = coord[atoms.index('SG')]    
+    SG = coord[atoms.index('SG')]
     atompos = [N, CA, C, O, CB, SG]
     atoms   = ['N', 'CA', 'C', 'O', 'CB', 'SG']
     return [atoms, atompos]
