@@ -1,7 +1,7 @@
 __doc__ = """
 This program requires python 3.6 or higher.
 
-Print(functionName.__doc__) to see the 
+Print(functionName.__doc__) to see the
 
 documentation for the function or usage.
 
@@ -10,13 +10,13 @@ Send enquiries to osita@protein-science.com
 import copy
 from . import LinearAlgebra
 from math import sqrt
-from . import PotentialEnergy 
+from . import PotentialEnergy
 
 def arginine_h(ires,i="",nextres=None):
     """
     This function plus the next 19 represent 20 amino acid residues.
-    Annotation is provided in detail here only  b/c all the 
-    functions are essentially the same, that is, they take 
+    Annotation is provided in detail here only  b/c all the
+    functions are essentially the same, that is, they take
     similar arguments except SER, THR, TYR and free CYS that have class6 H-atoms
     to be optimized with a potential energy function. The last 4 functions
     are different from these 20 and annotations are provided
@@ -41,21 +41,21 @@ def arginine_h(ires,i="",nextres=None):
     -------
     A list: index 0 is the atom coordinates,
             index 1 is backbone H if the next residue is not PRO
-            index 2 is the atom name, however, 
+            index 2 is the atom name, however,
             if the next residue is PRO, index 1 is the atom name
     """
     atoms,coord= ires[1], ires[2]
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_CD     = coord[atoms.index('CD')]
     pos_NE     = coord[atoms.index('NE')]
-    pos_CZ     = coord[atoms.index('CZ')] 
+    pos_CZ     = coord[atoms.index('CZ')]
     pos_NH1    = coord[atoms.index('NH1')]
-    pos_NH2    = coord[atoms.index('NH2')]                    
+    pos_NH2    = coord[atoms.index('NH2')]
 
     if nextres !=None:
         nextres_CA = nextres[2][1]
@@ -72,7 +72,7 @@ def arginine_h(ires,i="",nextres=None):
         HD1, HD2  =  LinearAlgebra.class2(pos_NE,pos_CG,pos_CD)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HE','1HH1','2HH1','1HH2','2HH2','HB1','HB2','HG1','HG2','HD1','HD2'])
         coord_copy.extend([HA,HE,HH11,HH12,HH21,HH22,HB1,HB2,HG1,HG2,HD1,HD2])
         return coord_copy, [H,i], atoms_copy
@@ -88,7 +88,7 @@ def arginine_h(ires,i="",nextres=None):
     HD1, HD2  =  LinearAlgebra.class2(pos_NE,pos_CG,pos_CD)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA','HE','1HH1','2HH1','1HH2','2HH2','HB1','HB2','HG1','HG2','HD1','HD2'])
     coord_copy.extend([HA,HE,HH11,HH12,HH21,HH22,HB1,HB2,HG1,HG2,HD1,HD2])
     return coord_copy, atoms_copy
@@ -96,7 +96,7 @@ def arginine_h(ires,i="",nextres=None):
 
 def isoleucine_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the ILE residue.
     See arginine() for full annotation
     """
@@ -104,7 +104,7 @@ def isoleucine_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG1    = coord[atoms.index('CG1')]
     pos_CG2    = coord[atoms.index('CG2')]
@@ -125,7 +125,7 @@ def isoleucine_h(ires,i="",nextres=None):
         HD3            =  LinearAlgebra.calcCoordinate(pos_CB,pos_CG1,pos_CD1,1.09, 60.0, 109.5)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB','1HG1','2HG1','1HG2','2HG2','3HG2','HD1','HD2','HD3'])
         coord_copy.extend([HA,HB,HG11,HG12,HG21,HG22,HG23,HD1,HD2,HD3])
         return coord_copy, [H,i], atoms_copy
@@ -141,14 +141,14 @@ def isoleucine_h(ires,i="",nextres=None):
     HD3            =  LinearAlgebra.calcCoordinate(pos_CB,pos_CG1,pos_CD1,1.09, 60.0, 109.5)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA','HB','1HG1','2HG1','1HG2','2HG2','3HG2','HD1','HD2','HD3'])
     coord_copy.extend([HA,HB,HG11,HG12,HG21,HG22,HG23,HD1,HD2,HD3])
     return coord_copy, atoms_copy
 
 def leucine_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the LEU residue.
     See arginine() for full annotation
     """
@@ -156,7 +156,7 @@ def leucine_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_CD1    = coord[atoms.index('CD1')]
@@ -177,7 +177,7 @@ def leucine_h(ires,i="",nextres=None):
         HD23           =  LinearAlgebra.calcCoordinate(pos_CB,pos_CG,pos_CD2, 1.09, 60.0,109.5)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2','HG','1HD1','2HD1','3HD1','1HD2','2HD2','3HD2'])
         coord_copy.extend([HA,HB1,HB2,HG,HD11,HD12,HD13,HD21,HD22,HD23])
         return coord_copy, [H,i], atoms_copy
@@ -193,14 +193,14 @@ def leucine_h(ires,i="",nextres=None):
     HD23           =  LinearAlgebra.calcCoordinate(pos_CB,pos_CG,pos_CD2,  1.09,60.0,109.5)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA','HB1','HB2','HG','1HD1','2HD1','3HD1','1HD2','2HD2','3HD2'])
     coord_copy.extend([HA,HB1,HB2,HG,HD11,HD12,HD13,HD21,HD22,HD23])
     return coord_copy, atoms_copy
 
 def tryptophan_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the TRP residue.
     See arginine() for full annotation
     """
@@ -208,7 +208,7 @@ def tryptophan_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_CD1    = coord[atoms.index('CD1')]
@@ -217,8 +217,8 @@ def tryptophan_h(ires,i="",nextres=None):
     pos_CE3    = coord[atoms.index('CE3')]
     pos_NE1    = coord[atoms.index('NE1')]
     pos_CZ2    = coord[atoms.index('CZ2')]
-    pos_CZ3    = coord[atoms.index('CZ3')] 
-    pos_CH2    = coord[atoms.index('CH2')] 
+    pos_CZ3    = coord[atoms.index('CZ3')]
+    pos_CH2    = coord[atoms.index('CH2')]
 
     if nextres !=None:
         nextres_CA = nextres[2][1]
@@ -234,7 +234,7 @@ def tryptophan_h(ires,i="",nextres=None):
         HH2            =  LinearAlgebra.class5(pos_CZ2,pos_CH2,pos_CZ3,1.08)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2','HD1','HE1','HE3','HZ2','HZ3','HH2'])
         coord_copy.extend([HA,HB1,HB2,HD1,HE1,HE3,HZ2,HZ3,HH2])
         return coord_copy, [H,i], atoms_copy
@@ -249,14 +249,14 @@ def tryptophan_h(ires,i="",nextres=None):
     HH2            =  LinearAlgebra.class5(pos_CZ2,pos_CH2,pos_CZ3,1.08)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA','HB1','HB2','HD1','HE1','HE3','HZ2','HZ3','HH2'])
     coord_copy.extend([HA,HB1,HB2,HD1,HE1,HE3,HZ2,HZ3,HH2])
     return coord_copy, atoms_copy
 
 def phenylalanine_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the PHE residue.
     See arginine() for full annotation
     """
@@ -264,7 +264,7 @@ def phenylalanine_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_CD1    = coord[atoms.index('CD1')]
@@ -286,7 +286,7 @@ def phenylalanine_h(ires,i="",nextres=None):
         HZ             =  LinearAlgebra.class5(pos_CE2,pos_CZ,pos_CE1,1.08)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2','HD1','HD2','HE1','HE2','HZ'])
         coord_copy.extend([HA,HB1,HB2,HD1,HD2,HE1,HE2,HZ])
         return coord_copy, [H,i], atoms_copy
@@ -300,14 +300,14 @@ def phenylalanine_h(ires,i="",nextres=None):
     HZ             =  LinearAlgebra.class5(pos_CE2,pos_CZ,pos_CE1,1.08)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA','HB1','HB2','HD1','HD2','HE1','HE2','HZ'])
     coord_copy.extend([HA,HB1,HB2,HD1,HD2,HE1,HE2,HZ])
     return coord_copy, atoms_copy
 
 def asparagine_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the ASN residue.
     See arginine() for full annotation
     """
@@ -315,7 +315,7 @@ def asparagine_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_ND2    = coord[atoms.index('ND2')]
@@ -331,7 +331,7 @@ def asparagine_h(ires,i="",nextres=None):
         HD22      =  LinearAlgebra.class4(pos_OD1,pos_CG,pos_ND2)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2','1HD2','2HD2'])
         coord_copy.extend([HA,HB1,HB2,HD12,HD22])
         return coord_copy, [H,i], atoms_copy
@@ -342,14 +342,14 @@ def asparagine_h(ires,i="",nextres=None):
     HD22      =  LinearAlgebra.class4(pos_OD1,pos_CG,pos_ND2)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA','HB1','HB2','1HD2','2HD2'])
     coord_copy.extend([HA,HB1,HB2,HD12,HD22])
     return coord_copy, atoms_copy
 
 def alanine_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the ALA residue.
     See arginine() for full annotation
     """
@@ -357,7 +357,7 @@ def alanine_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
 
     if nextres !=None:
@@ -370,7 +370,7 @@ def alanine_h(ires,i="",nextres=None):
         HB3            =  LinearAlgebra.calcCoordinate(pos_C,pos_CA,pos_CB,1.09,-64.2,109.5)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA', 'HB1', 'HB2', 'HB3'])
         coord_copy.extend([HA,HB1,HB2,HB3])
         return coord_copy,[H,i], atoms_copy
@@ -380,14 +380,14 @@ def alanine_h(ires,i="",nextres=None):
     HB2            =  LinearAlgebra.calcCoordinate(pos_C,pos_CA,pos_CB,1.09,175.8,109.5)
     HB3            =  LinearAlgebra.calcCoordinate(pos_C,pos_CA,pos_CB,1.09,-64.2,109.5)
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA', 'HB1', 'HB2', 'HB3'])
     coord_copy.extend([HA,HB1,HB2,HB3])
     return coord_copy, atoms_copy
 
 def glycine_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the GLY residue.
     See arginine() for full annotation
     """
@@ -395,7 +395,7 @@ def glycine_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
 
     if nextres !=None:
         nextres_CA = nextres[2][1]
@@ -404,21 +404,21 @@ def glycine_h(ires,i="",nextres=None):
         HA1,HA2   =  LinearAlgebra.class2(pos_C,pos_N,pos_CA)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA1', 'HA2'])
         coord_copy.extend([HA1,HA2])
         return coord_copy, [H,i], atoms_copy
 
     HA1,HA2   =  LinearAlgebra.class2(pos_C,pos_N,pos_CA)
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA1', 'HA2'])
     coord_copy.extend([HA1,HA2])
     return coord_copy, atoms_copy
 
 def valine_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the VAL residue.
     See arginine() for full annotation
     """
@@ -426,7 +426,7 @@ def valine_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG1    = coord[atoms.index('CG1')]
     pos_CG2    = coord[atoms.index('CG2')]
@@ -445,7 +445,7 @@ def valine_h(ires,i="",nextres=None):
         HG23           =  LinearAlgebra.calcCoordinate(pos_CG1,pos_CB,pos_CG2,1.09,-178.1,109.5)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB','1HG1','2HG1','3HG1','1HG2','2HG2','3HG2'])
         coord_copy.extend([HA,HB,HG11,HG12,HG13,HG21,HG22,HG23])
         return coord_copy, [H,i], atoms_copy
@@ -460,14 +460,14 @@ def valine_h(ires,i="",nextres=None):
     HG23           =  LinearAlgebra.calcCoordinate(pos_CG1,pos_CB,pos_CG2,1.09,-178.1,109.5)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA','HB','1HG1','2HG1','3HG1','1HG2','2HG2','3HG2'])
     coord_copy.extend([HA,HB,HG11,HG12,HG13,HG21,HG22,HG23])
     return coord_copy, atoms_copy
 
 def lysine_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the LYS residue.
     See arginine() for full annotation
     """
@@ -475,7 +475,7 @@ def lysine_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_CD     = coord[atoms.index('CD')]
@@ -496,7 +496,7 @@ def lysine_h(ires,i="",nextres=None):
         HZ3            =  LinearAlgebra.calcCoordinate(pos_CD,pos_CE,pos_NZ,1.01,60.0,109.5)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2','HG1','HG2','HD1','HD2','HE1','HE2','HZ1','HZ2','HZ3'])
         coord_copy.extend([HA,HB1,HB2,HG1,HG2,HD1,HD2,HE1,HE2,HZ1,HZ2,HZ3])
         return coord_copy, [H,i], atoms_copy
@@ -511,14 +511,14 @@ def lysine_h(ires,i="",nextres=None):
     HZ3            =  LinearAlgebra.calcCoordinate(pos_CD,pos_CE,pos_NZ,1.01,60.0,109.5)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA','HB1','HB2','HG1','HG2','HD1','HD2','HE1','HE2','HZ1','HZ2','HZ3'])
     coord_copy.extend([HA,HB1,HB2,HG1,HG2,HD1,HD2,HE1,HE2,HZ1,HZ2,HZ3])
     return coord_copy, atoms_copy
 
 def aspartate_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the ASP residue.
     See arginine() for full annotation
     """
@@ -526,7 +526,7 @@ def aspartate_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_OD1    = coord[atoms.index('OD1')]
@@ -540,7 +540,7 @@ def aspartate_h(ires,i="",nextres=None):
         HB1,HB2   = LinearAlgebra.class2(pos_CA,pos_CG,pos_CB)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA', 'HB1', 'HB2'])
         coord_copy.extend([HA,HB1,HB2])
         return coord_copy, [H,i], atoms_copy
@@ -549,14 +549,14 @@ def aspartate_h(ires,i="",nextres=None):
     HB1,HB2 = LinearAlgebra.class2(pos_CA,pos_CG,pos_CB)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA', 'HB1', 'HB2'])
     coord_copy.extend([HA,HB1,HB2])
     return coord_copy, atoms_copy
 
 def glutamine_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the GLN residue.
     See arginine() for full annotation
     """
@@ -564,7 +564,7 @@ def glutamine_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_CD     = coord[atoms.index('CD')]
@@ -582,7 +582,7 @@ def glutamine_h(ires,i="",nextres=None):
         HE22           =  LinearAlgebra.class4(pos_OE1,pos_CD,pos_NE2)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2','HG1','HG2','1HE2','2HE2'])
         coord_copy.extend([HA,HB1,HB2,HG1,HG2,HE21,HE22])
         return coord_copy, [H,i], atoms_copy
@@ -594,14 +594,14 @@ def glutamine_h(ires,i="",nextres=None):
     HE22           =  LinearAlgebra.class4(pos_OE1,pos_CD,pos_NE2)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA','HB1','HB2','HG1','HG2','1HE2','2HE2'])
     coord_copy.extend([HA,HB1,HB2,HG1,HG2,HE21,HE22])
     return coord_copy, atoms_copy
 
 def glutamate_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the GLU residue.
     See arginine() for full annotation
     """
@@ -609,7 +609,7 @@ def glutamate_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_CD     = coord[atoms.index('CD')]
@@ -625,7 +625,7 @@ def glutamate_h(ires,i="",nextres=None):
         HG1,HG2        =  LinearAlgebra.class2(pos_CB,pos_CD,pos_CG)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2','HG1','HG2'])
         coord_copy.extend([HA,HB1,HB2,HG1,HG2])
         return coord_copy, [H,i], atoms_copy
@@ -635,14 +635,14 @@ def glutamate_h(ires,i="",nextres=None):
     HG1,HG2        =  LinearAlgebra.class2(pos_CB,pos_CD,pos_CG)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA','HB1','HB2','HG1','HG2'])
     coord_copy.extend([HA,HB1,HB2,HG1,HG2])
     return coord_copy, atoms_copy
 
 def histidine_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the HIS residue.
     See arginine() for full annotation
     """
@@ -650,7 +650,7 @@ def histidine_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_CD2    = coord[atoms.index('CD2')]
@@ -669,7 +669,7 @@ def histidine_h(ires,i="",nextres=None):
         HE2            =  LinearAlgebra.class5(pos_CD2,pos_NE2,pos_CE1,1.01)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2','HD2','HE1','HE2'])
         coord_copy.extend([HA,HB1,HB2,HD2,HE1,HE2])
         return coord_copy, [H,i], atoms_copy
@@ -681,14 +681,14 @@ def histidine_h(ires,i="",nextres=None):
     HE2            =  LinearAlgebra.class5(pos_CD2,pos_NE2,pos_CE1,1.01)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA','HB1','HB2','HD2','HE1','HE2'])
     coord_copy.extend([HA,HB1,HB2,HD2,HE1,HE2])
     return coord_copy, atoms_copy
 
 def proline_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the PRO residue.
     See arginine() for full annotation
     """
@@ -696,7 +696,7 @@ def proline_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_CD     = coord[atoms.index('CD')]
@@ -711,7 +711,7 @@ def proline_h(ires,i="",nextres=None):
         HD1,HD2        =  LinearAlgebra.class2(pos_CG,pos_N,pos_CD)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA', 'HB1', 'HB2', 'HG1', 'HG2', 'HD1', 'HD2'])
         coord_copy.extend([HA,HB1,HB2,HG1,HG2,HD1,HD2])
         return coord_copy,[H,i], atoms_copy
@@ -723,7 +723,7 @@ def proline_h(ires,i="",nextres=None):
     HD1,HD2        =  LinearAlgebra.class2(pos_CG,pos_N,pos_CD)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA', 'HB1', 'HB2', 'HG1', 'HG2', 'HD1', 'HD2'])
     coord_copy.extend([HA,HB1,HB2,HG1,HG2,HD1,HD2])
     return coord_copy, atoms_copy
@@ -731,7 +731,7 @@ def proline_h(ires,i="",nextres=None):
 
 def metheonine_h(ires,i="",nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the MET residue.
     See arginine() for full annotation
     """
@@ -739,7 +739,7 @@ def metheonine_h(ires,i="",nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_SD     = coord[atoms.index('SD')]
@@ -757,7 +757,7 @@ def metheonine_h(ires,i="",nextres=None):
         HE3            =  LinearAlgebra.calcCoordinate(pos_CG,pos_SD,pos_CE,1.09,60.0,109.5)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2','HG1','HG2','HE1','HE2','HE3'])
         coord_copy.extend([HA,HB1,HB2,HG1,HG2,HE1,HE2,HE3])
         return coord_copy, [H,i], atoms_copy
@@ -770,14 +770,14 @@ def metheonine_h(ires,i="",nextres=None):
     HE3            =  LinearAlgebra.calcCoordinate(pos_CG,pos_SD,pos_CE,1.09,60.0,109.5)
 
     atoms_copy = copy.deepcopy(atoms)
-    coord_copy = copy.deepcopy(coord) 
+    coord_copy = copy.deepcopy(coord)
     atoms_copy.extend(['HA','HB1','HB2','HG1','HG2','HE1','HE2','HE3'])
     coord_copy.extend([HA,HB1,HB2,HG1,HG2,HE1,HE2,HE3])
     return coord_copy, atoms_copy
 
 def isDisulfide(ires,i,nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the CYS (Cystine) residue.
     See arginine() for full annotation
     """
@@ -785,7 +785,7 @@ def isDisulfide(ires,i,nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_SG     = coord[atoms.index('SG')]
 
@@ -797,7 +797,7 @@ def isDisulfide(ires,i,nextres=None):
         HB1,HB2        =  LinearAlgebra.class2(pos_CA,pos_SG,pos_CB)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2'])
         coord_copy.extend([HA,HB1,HB2])
         return coord_copy, [H,i], atoms_copy
@@ -806,14 +806,14 @@ def isDisulfide(ires,i,nextres=None):
         HA             =  LinearAlgebra.class3(pos_CB,pos_N,pos_CA)
         HB1,HB2        =  LinearAlgebra.class2(pos_CA,pos_SG,pos_CB)
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2'])
         coord_copy.extend([HA,HB1,HB2])
         return coord_copy, atoms_copy
 
 def notDisulfide(ires,i,resNo,atoms2,atmpos2,resNo2,resseq2,nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the CYS (Cysteine) residue.
     See arginine() for full annotation
     """
@@ -821,7 +821,7 @@ def notDisulfide(ires,i,resNo,atoms2,atmpos2,resNo2,resseq2,nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_SG     = coord[atoms.index('SG')]
 
@@ -837,7 +837,7 @@ def notDisulfide(ires,i,resNo,atoms2,atmpos2,resNo2,resseq2,nextres=None):
         HG            =  PotentialEnergy.optmizeH(resNo,atoms2,atmpos2,resNo2,resseq2,xyz)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2','HG'])
         coord_copy.extend([HA,HB1,HB2,HG])
         return coord_copy, [H,i], atoms_copy
@@ -851,14 +851,14 @@ def notDisulfide(ires,i,resNo,atoms2,atmpos2,resNo2,resseq2,nextres=None):
         HG            =  PotentialEnergy.optmizeH(resNo,atoms2,atmpos2,resNo2,resseq2,xyz)
 
         atoms_copy = copy.deepcopy(atoms)
-        coord_copy = copy.deepcopy(coord) 
+        coord_copy = copy.deepcopy(coord)
         atoms_copy.extend(['HA','HB1','HB2','HG'])
         coord_copy.extend([HA,HB1,HB2,HG])
         return coord_copy, atoms_copy
 
 def threonine_h(ires,i,resNo,atoms2,atmpos2,resNo2,resseq2,nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the THR residue.
     See arginine() for full annotation
     """
@@ -866,7 +866,7 @@ def threonine_h(ires,i,resNo,atoms2,atmpos2,resNo2,resseq2,nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG2    = coord[atoms.index('CG2')]
     pos_OG1    = coord[atoms.index('OG1')]
@@ -911,7 +911,7 @@ def threonine_h(ires,i,resNo,atoms2,atmpos2,resNo2,resseq2,nextres=None):
 
 def serine_h(ires,i,resNo,atoms2,atmpos2,resNo2,resseq2,nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the SER residue.
     See arginine() for full annotation
     """
@@ -919,7 +919,7 @@ def serine_h(ires,i,resNo,atoms2,atmpos2,resNo2,resseq2,nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_OG     = coord[atoms.index('OG')]
 
@@ -956,7 +956,7 @@ def serine_h(ires,i,resNo,atoms2,atmpos2,resNo2,resseq2,nextres=None):
 
 def tyrosine_h(ires,i,resNo,atoms2,atmpos2,resNo2,resseq2,nextres=None):
     """
-    This function adds missing hydrogen 
+    This function adds missing hydrogen
     atoms of the TYR residue.
     See arginine() for full annotation
     """
@@ -964,15 +964,15 @@ def tyrosine_h(ires,i,resNo,atoms2,atmpos2,resNo2,resseq2,nextres=None):
     pos_N      = coord[atoms.index('N')]
     pos_CA     = coord[atoms.index('CA')]
     pos_C      = coord[atoms.index('C')]
-    pos_O      = coord[atoms.index('O')]    
+    pos_O      = coord[atoms.index('O')]
     pos_CB     = coord[atoms.index('CB')]
     pos_CG     = coord[atoms.index('CG')]
     pos_CD1    = coord[atoms.index('CD1')]
     pos_CD2    = coord[atoms.index('CD2')]
     pos_CE1    = coord[atoms.index('CE1')]
     pos_CE2    = coord[atoms.index('CE2')]
-    pos_CZ     = coord[atoms.index('CZ')] 
-    pos_OH     = coord[atoms.index('OH')] 
+    pos_CZ     = coord[atoms.index('CZ')]
+    pos_OH     = coord[atoms.index('OH')]
 
     if nextres !=None:
         nextres_CA = nextres[2][1]
@@ -1084,18 +1084,41 @@ def ntermini_pro(ires,resn2):
     coord      = ires[2]
     atoms,coord= ires[1], ires[2]
     pos_N      = coord[atoms.index('N')]
-    pos_CA     = coord[atoms.index('CA')]   
+    pos_CA     = coord[atoms.index('CA')]
     pos_CD     = coord[atoms.index('CD')]
 
     H1,H2 = LinearAlgebra.class2(pos_CA,pos_CD,pos_N,1.01)
 
     return [H1,H2]
 
-def his_prot(ires):
+def his_note(resNo, chainNo):
     """
-    This function adds missing hydrogen 
-    atoms of the HIS residue.
-    See arginine() for full annotation
+    This function informs user
+    whenever HIS is protonated
+    on both delta and epsilon N
+
+    Arguments
+    ----------
+    resNo:   the residue number
+
+    chainNo: the chain number
+
+    Returns
+    -------
+    None:   writes to a log.txt file
+
+    """
+    if resNo[-1].isalpha():
+        resNo = resNo[:-1]
+    with open('log.txt', 'a') as f:
+        f.write('HIS {} in chain {} is protonated (+1 charge)'\
+        .format(resNo,chainNo)+'\n\n')
+
+def get_hd1(ires):
+    """
+    This function adds hydrogen atom to ND1 of
+    histidine making it to have +1 charge.
+    input argument has the same meaning as previous
     """
     atoms,coord= ires[1], ires[2]
     pos_CG     = coord[atoms.index('CG')]
@@ -1103,4 +1126,36 @@ def his_prot(ires):
     pos_CE1    = coord[atoms.index('CE1')]
 
     HD1        =  LinearAlgebra.class5(pos_CE1,pos_ND1,pos_CG,1.01)
-    return [HD1]
\ No newline at end of file
+    return [HD1]
+
+def prot_his(x,res_pos,atom_name,chainNo):
+    """
+    This function checks if there are more than 4 HIS resi in order to add
+    # hydrogen atom to ND1 of histidine making it to have +1 charge.
+
+    ----------
+    x:       a list containing all residue name,atoms,atom coordinates
+
+    res_pos: a list containing all residue atom positions
+
+    atom_name: a list containing all residue atom names
+
+    chainNo:   the chain number of the chain
+
+    Returns
+    -------
+    A list:   index 0 = updated res_pos and index 1 = updated atom_name
+    """
+    his  = [j for i,j in enumerate(x) if x[i][0][:3] == 'HIS']
+    num  = [res_pos[i] for i,j in enumerate(x) if x[i][0][:3] == 'HIS']
+    name = [atom_name[i] for i,j in enumerate(x) if x[i][0][:3] == 'HIS']
+    if len(his) <= 4:
+        pass
+    else:
+        prot = len(his)//5
+        for i in range(prot):
+            res_pos[res_pos.index(num[i])].extend(get_hd1(his[i]))
+            atom_name[atom_name.index(name[i])].extend(['HD1'])
+            his_note(his[i][0][3:],chainNo+1)
+
+    return [res_pos, atom_name]
\ No newline at end of file
