@@ -1,7 +1,7 @@
 __doc__ = """
 This program requires python 3.6 or higher.
 
-Print(functionName.__doc__) to see the 
+Print(functionName.__doc__) to see the
 
 documentation for the function or usage.
 
@@ -13,41 +13,20 @@ import itertools
 from .MissingHydrogenAtoms import *
 from .CheckPDBatoms import checkpdbAtoms
 
-def his_note(resNo, chainNo):
-    """
-    This function informs user
-    whenever HIS is protonated
-    on both delta and epsilon N
-    
-    Arguments
-    ----------
-    resNo:   the residue number
-
-    chainNo: the chain number
-    
-    Returns
-    -------
-    None:   writes to a log.txt file
-     
-    """
-    with open('log.txt', 'a') as f:
-        f.write('HIS {} in chain {} is protonated (+1 charge)'\
-        .format(resNo,chainNo)+'\n\n')
-
 def atomType(name):
     """
     This function returns the
     format for writing the atom
     name to a PDB file
-    
+
     Arguments
     ----------
     name: the standard PDB atom name
-    
+
     Returns
     -------
-    string: the atom name and format 
-            for writing to a PDB file 
+    string: the atom name and format
+            for writing to a PDB file
     """
     if len(name) == 1:
         name = " %s  " % name
@@ -57,24 +36,24 @@ def atomType(name):
         name = " %s" % name
     elif len(name) == 4:
         name = "%s" % name
-    return name 
+    return name
 
 def insertRes(res):
     """
     This function returns the
     letter for residue insertion
     to be written to a PDB file
-    
+
     Arguments
     ----------
     res: the res name + res No + insertion
          as concatenated by PRAS when reading
          input PDB file
-    
+
     Returns
     -------
-    string: the insertion code/letter if it 
-            exists else a blank space 
+    string: the insertion code/letter if it
+            exists else a blank space
     """
     if res[-1].isalpha():
         return res[-1]
@@ -86,31 +65,31 @@ def repairPDB(pdb_pras,rotamer,mutation,pdb_faspr):
     This function steps over each residue of a chain and obtains from
     MissingHydrogenAtoms.py all atoms of the residue to be wrttien to a new PDB file.
 
-    If the residue is not the last and the next residue is not PRO the returned list 
-    will contain a backbone H otherwise it will have no backbone H. 
+    If the residue is not the last and the next residue is not PRO the returned list
+    will contain a backbone H otherwise it will have no backbone H.
 
     For Cys residue, SG_coords are used to check for disulfide bonds.
     If disulfide bond exists it obtains no HG, otherwise it obtains HG
-    
+
     Arguments (this function takes 4 compulsory arguments)
     ----------
     pdb_pras:  the syntactically correct PDB file to repair/analyze
 
     rotamer:   by default, atoms with the highest occupancy are
                written to PDB file in the case of rotamers.
-               If you want low occupancy instead, supply "no" or any 
+               If you want low occupancy instead, supply "no" or any
                string as the argument, otherwise supply "" as argument.
 
     mutation:  when there is point mutation, two residues with unequal
-               occupany (i.e. the residue atoms) are given same residue number. 
+               occupany (i.e. the residue atoms) are given same residue number.
                By default the residue with the highest occupancy is
-               written to PDB file. If you want low occupancy instead supply "no" 
+               written to PDB file. If you want low occupancy instead supply "no"
                or any string as the argument, otherwise supply "" as argument.
 
-    pdb_faspr: the PDB file obtained from FASPR (by running the same PDB supplied to PRAS). 
+    pdb_faspr: the PDB file obtained from FASPR (by running the same PDB supplied to PRAS).
                FASPR is a free and open source side-chain packer, thanks to Huang et al.
                You can obtain it at https://github.com/tommyhuangthu/FASPR
-               
+
                Note that if there are no missing atoms that require flexible
                chi to fix then FASPR is not required. If there are such
                atoms and you do not supply FASPR PDB file, PRAS will use
@@ -119,7 +98,7 @@ def repairPDB(pdb_pras,rotamer,mutation,pdb_faspr):
                notify you if FASPR PDB is not supplied.
 
                Although FASPR is more accurate than most state-of-the-art side-chain packers,
-               it is not infinitely accurate and sometimes the default chi from PRAS is the 
+               it is not infinitely accurate and sometimes the default chi from PRAS is the
                right conformation for the amino acid residue side-chain. It is adviced that
                you compare both methods when necessary.
 
@@ -130,15 +109,15 @@ def repairPDB(pdb_pras,rotamer,mutation,pdb_faspr):
           run the program
     """
     try:
-        os.remove('out_with_h.pdb') 
+        os.remove('out_with_h.pdb')
     except:
         pass
 
-    nchains = checkpdbAtoms(pdb_pras,rotamer,mutation,pdb_faspr) 
+    nchains = checkpdbAtoms(pdb_pras,rotamer,mutation,pdb_faspr)
     chains = list(itertools.chain(*[i[-1] for i in nchains]))
 
     # atom numbering starts from the first chain
-    number=1 
+    number=1
     for n in range(len(nchains)):
         resn,atom,atmpos,resNo,resseq,sg_coord,chain = nchains[n]
         x = list(zip(resn,atom,atmpos))
@@ -148,7 +127,7 @@ def repairPDB(pdb_pras,rotamer,mutation,pdb_faspr):
         # it returns backbone H but if PRO or the last residue in the
         # chain it does not return backbone H. The H is later added
         # to residue i+1
-        for i,j in  enumerate(x): 
+        for i,j in  enumerate(x):
             if j[0].startswith('ARG'):
                 if x[(i+1)% len(x)][0][:3] != 'PRO' and x[i] != x[-1]:
                     apos,hpos,name = arginine_h(x[i],i,x[(i+1)%len(x)])
@@ -279,9 +258,9 @@ def repairPDB(pdb_pras,rotamer,mutation,pdb_faspr):
 
                     """
                     The remaining residues (SER, THR, TYR and free CYS) have what we regard as
-                    class6 H-atoms with rotational freedom. These H-atoms are optimized with a 
+                    class6 H-atoms with rotational freedom. These H-atoms are optimized with a
                     potential energy function that may affect the computational time. You can go
-                    to PotentialEnergy.py and look for "optimize()" function and decrease/increase 
+                    to PotentialEnergy.py and look for "optimize()" function and decrease/increase
                     the rotation interval. Currently, it is set to 45 degrees but in the online PRAS
                     server it is set to a much lower value.
 
@@ -329,18 +308,12 @@ def repairPDB(pdb_pras,rotamer,mutation,pdb_faspr):
                         res_pos.extend([apos])
                         atom_name.extend([name])
 
+        ##################################################################################################################
+        # comment next code line if you want all HIS side-chain neutral. See explanation online at www.protein-science.com
         #check HIS protonation
-        his = [j for i,j in enumerate(x) if x[i][0][:3] == 'HIS']
-        if len(his) <= 4:
-            pass
-        else:
-            prot = len(his)//5
-            for i in range(prot):
-                res_pos[int(his[i][0][3:])-1].extend(his_prot(his[i]))
-                atom_name[int(his[i][0][3:])-1].extend(['HD1'])
-                his_note(int(his[i][0][3:]),n+1)
+        res_pos, atom_name = prot_his(x,res_pos,atom_name,n)
 
-        # Add backbone hydrogen. 
+        # Add backbone hydrogen.
         # It belongs to resi i+1
         for i,j in enumerate(h_pos):
             res_pos[h_pos[i][1]+1].extend([h_pos[i][0]])
@@ -354,14 +327,14 @@ def repairPDB(pdb_pras,rotamer,mutation,pdb_faspr):
         else:
             res_pos[0].extend(ntermini_notpro(x[0],resn[0][:3]))
             atom_name[0].extend(['H1','H2','H3'])
-        
+
         # steps over each residue
-        # then steps over each residue atom  
+        # then steps over each residue atom
         # and writes to a PDB file
-        with open('out_with_h.pdb', 'a')  as f:            
-            for k,l in enumerate (res_pos): 
+        with open('out_with_h.pdb', 'a')  as f:
+            for k,l in enumerate (res_pos):
                 atm = atom_name[k]
-                for i,j in enumerate(l): 
+                for i,j in enumerate(l):
                     f.write("%6s%5s %4s %-4s%1s%4s%1s   %8.3f%8.3f%8.3f%6.2f%6.2f" \
                     %('ATOM  ',str(number)[-5:],atomType(atm[i]),resseq[k],\
                     chains[n],str(resNo[k])[-4:],insertRes(resn[k]),\
@@ -372,7 +345,7 @@ def repairPDB(pdb_pras,rotamer,mutation,pdb_faspr):
         # write TER at the end and if the last chain
         # write TER and then END
         with open('out_with_h.pdb', 'a')  as f:
-            if n != len(nchains)-1: 
+            if n != len(nchains)-1:
                 f.write('{:4s}' '{:2s}' '{:5d}''{:s}' '{:4s}''{:s}''{:3s}' '{:2s}''{:4d}\n'.format\
                     ('TER',' ',number,' ',' ', ' ',resseq[len(x)-1],' '+chains[n],resNo[len(x)-1]))
             else:
